const { Builder } = require('selenium-webdriver')
const fs = require('fs')

const driver = new Builder().forBrowser('chrome').build()

/**
 * 在电影列表页面获取每部电影内容页面链接
 * @returns string[] 链接数组
 */
const getMovieLinks = () => Array.from(document.querySelectorAll('.grid_view .info a')).map(link => link.href)

/**
 * 在电影内容页面获取电影信息
 * @returns {year: string, director: string, leading: string[], name: string, rating: {average: string, count: string, weights: string[]}, categories: string} 电影信息
 */
const getMovieData = (callback) => {
  let finished = false
  const getData = () => {
    if (finished) return
    const $rating = document.querySelector('#interest_sectl')
    // onload和setTimeout是异步执行的，要用callback替换return实现返回
    callback({
      'name': document.querySelector('[property="v:itemreviewed"]').innerText,
      'year': document.querySelector('.year').innerText.substr(1, 4),
      'rating': {
        'average': $rating.querySelector('[property="v:average"]').innerText,
        'count': $rating.querySelector('[property="v:votes"]').innerText,
        'weights': Array.from($rating.querySelectorAll('.rating_per')).map((rating) => rating.innerText)
      },
      'director': document.querySelector('#info span:first-child .attrs').innerText,
      'leading': Array.from(document.querySelectorAll('#celebrities .celebrity a.name')).map((leading) => leading.innerText),
      'categories': document.querySelector('#info [property="v:genre"]').innerText,
    })
    finished = true
  }
  // 处理页面未加载完成就执行getData引起的报错
  try {
    getData()
  } catch (e) {
    onload = getData
  }
  // 极小几率会出现上面代码不执行的情况，延迟两秒执行保底
  setTimeout(getData, 2000)
}


;(async () => {
  /**
   * 访问电影列表页面，在里面执行getMovieLinks函数获取每部电影的链接
   */
  const links = [];
  for (let i = 0; i < 250; i += 25) {
    await driver.get(`https://movie.douban.com/top250?start=${i}`)
    links.push(...await driver.executeScript(getMovieLinks))
    console.log(`getting links ${i}`)
  }
  /**
   * 分别访问上面获取到的链接，执行getData获取电影信息
   */
  const movies = [];
  for (let i = 0, len = links.length; i < len; i ++) {
    await driver.get(links[i])
    const movie = await driver.executeAsyncScript(getMovieData)
    movie.rank = i + 1
    movies.push(movie)
    console.log(movie)
  }
  /**
   * 抓取的信息写入到json文件中，并退出浏览器
   */
  fs.writeFileSync('top250.json', JSON.stringify(movies))
  await driver.quit()
})()
